var createError = require('http-errors');
var express = require('express');
var path = require('path');
var cookieParser = require('cookie-parser');
var logger = require('morgan');
var flash = require('connect-flash');

const methodOverride = require('method-override')

var { graphqlHTTP } = require('express-graphql');


var usersRouter = require('./routes/users');
var indexRouter = require('./routes/index');

var schema = require('./graphql/index')



//session passport and...
const passport			= require('passport');
const session			= require('express-session');
var app = express();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'twig');



app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));
//session
app.use(flash());
app.use(session({
	secret: "verygoodsecret",
	resave: false,
	saveUninitialized: false
}));


// Passport.js
app.use(passport.initialize());
app.use(passport.session());
app.use(methodOverride('_method'))

app.use('/', indexRouter);
app.use('/users', usersRouter);

//___________________________
//JWT
const authenticate  = require("./middleware/auth")
app.use(authenticate)


const {
  GraphQLInt,
  GraphQLString,
  GraphQLList,
  GraphQLObjectType,
} = require('graphql');


app.use('/graphql', graphqlHTTP({
  schema: schema,
  graphiql: true,
}));
//___________________________

// db
const db = require("./models");
db.mongoose
  .connect(db.url, {
    useNewUrlParser: true,
    useUnifiedTopology: true
  })
  .then(() => {
    console.log("Connected to the database!");
  })
  .catch(err => {
    console.log("Cannot connect to the database!", err);
    process.exit();
  });





// catch 404 and forward to error handler
app.use(function(req, res, next) {
  next(createError(404));
});

// error handler
app.use(function(err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

module.exports = app;
